from pathlib import Path
from watchdog.observers import Observer

from asset_discovery.file_monitor.any_file_changed_handler import AnyFileChangedHandler
from asset_discovery.file_monitor.start_monitoring_response import StartMonitoringResponse
from asset_discovery.notification.callback_registry import CallbackRegistry


class FileMonitor:
    def __init__(self, path, file_name=None, file_extension=None, callback_registry=None):
        self._path = path
        self._callback_registry = callback_registry if callback_registry is not None else CallbackRegistry()
        file_extension = file_extension if file_extension is not None else ".json"
        patterns = ["*"+file_name] if file_name is not None else ["*" + file_extension]
        file_changed_event_handler = AnyFileChangedHandler(self._callback_registry, patterns=patterns)
        self._observer = Observer()
        self._observer.setDaemon(True)
        self._observer.schedule(file_changed_event_handler, self._path, recursive=True)
        self._monitoring_started = False

    def register_change_callback(self, callback_id, change_callback, *args):
        self._callback_registry.register_change_callback(callback_id, change_callback, *args)

    def start_monitoring(self):
        if self._monitoring_started:
            return StartMonitoringResponse.MonitoringAlreadyStarted
        self._ensure_monitored_dir_exists()
        self._observer.start()
        self._monitoring_started = True
        return StartMonitoringResponse.Successful

    def _ensure_monitored_dir_exists(self):
        Path(self._path).mkdir(parents=True, exist_ok=True)

    def stop_monitoring(self):
        if self._monitoring_started and self._observer is not None:
            self._observer.stop()
            self._observer.join()
            self._observer = None
