import pprint

from datetime import datetime


class LifetimePowerOn:

    property_types = {
        'power_on_hours': 'int',
        'timestamp': 'datetime'
    }

    serialization_keys = {
        'power_on_hours': 'powerOnHours',
        'timestamp': 'timestamp'
    }

    def __init__(self,
                 power_on_hours=None,
                 timestamp=None):
        self._power_on_hours = power_on_hours
        self._timestamp = timestamp

    @property
    def power_on_hours(self) -> int:
        """Gets the power_on_hours of an asset.

        Lifetime power on hours.

        :return: The power_on_hours.
        :rtype: int
        """
        return self._power_on_hours

    @power_on_hours.setter
    def power_on_hours(self, power_on_hours) -> None:
        """Sets the power_on_hours of an asset.

        Lifetime power on hours.

        :param power_on_hours: The lifetime power on hours.
        :type: int
        """
        self._power_on_hours = power_on_hours

    @property
    def timestamp(self) -> datetime:
        return self._timestamp

    @timestamp.setter
    def timestamp(self, timestamp) -> None:
        self._timestamp = timestamp

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.property_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, LifetimePowerOn):
            return False

        return (self.power_on_hours == other.power_on_hours and
                self.timestamp == other.timestamp)

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self.__eq__(other)
