# coding: utf-8
"""
    Model representing a location.
"""

import pprint


class Location:
    """
    Model representing a location.
    """
    property_types = {
        'parent': 'str',
        'resource_uri': 'str',
        'slot_number': 'int',
        'state': 'AssetPresence'
    }

    serialization_keys = {
        'parent': 'parent',
        'resource_uri': 'resourceUri',
        'slot_number': 'slotNumber',
        'state': 'state'
    }

    def __init__(self,
                 parent=None,
                 resource_uri=None,
                 slot_number=None,
                 state=None):
        self._parent = None
        self._resource_uri = None
        self._slot_number = None
        self._state = None

        if parent is not None:
            self.parent = parent
        if resource_uri is not None:
            self.resource_uri = resource_uri
        if slot_number is not None:
            self.slot_number = slot_number
        if state is not None:
            self.state = state

    @property
    def parent(self):
        """Gets the parent of this Location.

        The parent of the asset.

        :return: The parent of this Location.
        :rtype: str
        """
        return self._parent

    @parent.setter
    def parent(self, parent):
        """Sets the parent of this Location.

        The parent of the asset.

        :param parent: The parent of this Location.
        :type: str
        """

        self._parent = parent

    @property
    def resource_uri(self):
        """Gets the resource_uri of this Location.

        Identifier of a resource.

        :return: The resource_uri of this Location.
        :rtype: str
        """
        return self._resource_uri

    @resource_uri.setter
    def resource_uri(self, resource_uri):
        """Sets the resource_uri of this Location.

        Identifier of a resource.

        :param resource_uri: The resource_uri of this Location.
        :type: str
        """

        self._resource_uri = resource_uri

    @property
    def slot_number(self):
        """Gets the slot_number of this Location.

        The number of the slot in which the asset is located.

        :return: The slot_number of this Location.
        :rtype: int
        """
        return self._slot_number

    @slot_number.setter
    def slot_number(self, slot_number):
        """Sets the slot_number of this Location.

        The number of the slot in which the asset is located.

        :param slot_number: The slot_number of this Location.
        :type: int
        """

        self._slot_number = slot_number

    @property
    def state(self):
        """Gets the state of this Location.


        :return: The state of this Location.
        :rtype: AssetPresence
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this Location.


        :param state: The state of this Location.
        :type: AssetPresence
        """

        self._state = state

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.property_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Location):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self.__eq__(other)
