"""
National Instruments SystemLink Assets JSON Util Module
"""
import json
try:
    import tornado as tornado
except ImportError:
    import salt.ext.tornado as tornado
from asset_discovery.discovery_engine_client import DiscoveryEngineClient


def list_asset_identifications(**kwargs):
    """
    Lists asset identification information in JSON format.
    """

    discovery_engine_client = kwargs.get('discovery_engine_client', None)
    if not isinstance(discovery_engine_client, DiscoveryEngineClient):
        discovery_engine_client = DiscoveryEngineClient()
    if discovery_engine_client is None:
        return json.dumps([]), -1
    assets = discovery_engine_client.list_assets(**kwargs)
    assets = _sanitize_assets_identification(assets)
    val = [_map_asset_identification(a) for a in assets if _has_valid_identification(a)]
    return json.dumps(val), 0


def _sanitize_assets_identification(assets):
    sanitized_assets = []
    for asset in assets:
        if asset.model_name is None:
            asset.model_name = ''
        if asset.model_number is None:
            asset.model_number = 0
        if asset.serial_number is None:
            asset.serial_number = ''
        if asset.vendor_name is None:
            asset.vendor_name = ''
        if asset.vendor_number is None:
            asset.vendor_number = 0
        if asset.name is None:
            asset.name = ''
        sanitized_assets.append(asset)
    return sanitized_assets


def _map_asset_identification(asset):
    asset_identification_dict = asset.identification.to_dict()
    asset_identification_dict["name"] = asset.name
    return asset_identification_dict


def _has_valid_identification(asset):
    return (asset.model_number != 0 or asset.model_name != '')\
           and (asset.vendor_number != 0 or asset.vendor_name != '')\
           and asset.serial_number != ''\
           and asset.name != ''
