from copy import deepcopy
from typing import Any, Tuple

from asset_discovery.models import Asset, OperationResult, OperationStatus
from asset_discovery.plugins.sysapi_imports.request_helpers.add.add_request_builder_base import AddRequestBuilderBase


class AddNetworkDaqmxRequestBuilder(AddRequestBuilderBase):
    """
    "timeout" -> optional, default 10000 (ms)
    "ip_address" and/or "hostname"
    """
    def __init__(self):
        self._base_request = {
            "api": "daqmx",
            "payload": {
                "timeout": 10000
            }
        }

    def build_from(self, asset: Asset, **kwargs) -> OperationResult:
        request = deepcopy(self._base_request)

        ip_address = self._get_first_ip_address(asset)

        if ip_address is not None:
            request["payload"]["ip_address"] = ip_address
        if ip_address is None:
            hostname = asset.hostname
            if hostname is not None:
                request["payload"]["hostname"] = hostname

        timeout = kwargs.get("timeout", None)
        if timeout is not None:
            is_int, timeout = self._is_int(timeout)
            if is_int:
                request["payload"]["timeout"] = timeout
            else:
                status = OperationStatus(
                    successful=False,
                    code=-1,
                    description="Parameter \"timeout\" should be an integer."
                )
                return OperationResult(status=status)

        status = OperationStatus(successful=True, code=0)
        return OperationResult(status=status, content=request)

    @classmethod
    def _is_int(cls, val: Any) -> Tuple[bool, int]:
        try:
            float_val = float(val)
            is_int = float_val.is_integer()
            if not is_int:
                return False, 0
            return True, int(float_val)
        except ValueError:
            return False, 0
