from copy import deepcopy

from asset_discovery.models import Asset, OperationResult, OperationStatus
from asset_discovery.plugins.sysapi_imports.repository.models import AttributeTag
from asset_discovery.plugins.sysapi_imports.request_helpers.add.add_request_builder_base import AddRequestBuilderBase


class AddNetworkVisaRequestBuilder(AddRequestBuilderBase):
    """
    {
        "validate_only" -> optional, default False,
        "target_uri" -> optional, default //localhost
        "ip_address" -> at least one of ip_address/hostname
        "hostname"
    }
    """
    def __init__(self):
        self._base_request = {
            "api": "sysapi",
            "payload": {
                "validate_only": False,
                "target_uri": "//localhost",
                "input_bag": {
                    "properties": [
                        {
                            # Item is device
                            "tag": AttributeTag.ItemIsDevice,
                            "type": "bool",
                            "value": True
                        },
                        {
                            # Bus type TCP/IP
                            "tag": AttributeTag.ConnectsToBusType,
                            "type": "uint",
                            "value": 6
                        }
                    ]
                }
            }
        }

        self._network_identification_property = {
            # TcpPrimaryHostname (Hostname or IP)
            "tag": AttributeTag.TcpPrimaryHostname,
            "type": "string",
            "value": ""
        }

    def build_from(self, asset: Asset, **kwargs) -> OperationResult:
        request = deepcopy(self._base_request)

        validate_only = kwargs.get("validate_only", None)
        if validate_only is not None:
            request["payload"]["validate_only"] = validate_only

        target_uri = kwargs.get("target_uri", None)
        if target_uri is not None:
            request["payload"]["target_uri"] = target_uri

        network_identification = deepcopy(self._network_identification_property)
        ip_address = self._get_first_ip_address(asset)
        if ip_address is not None:
            network_identification["value"] = ip_address
        else:
            hostname = asset.hostname
            network_identification["value"] = hostname
        request["payload"]["input_bag"]["properties"].append(network_identification)

        status = OperationStatus(successful=True, code=0)
        return OperationResult(status=status, content=request)
