from asset_discovery.exceptions import NotSupportedError


class PluginProxy:
    def __init__(self, id_, weight, merge, instance):
        self._id = id_
        self._weight = weight
        self._merge = merge
        self._instance = instance

    @property
    def id(self):
        return self._id

    @property
    def weight(self):
        return self._weight

    @property
    def merge(self):
        return self._merge

    def list_assets(self, refresh_reason, **kwargs):
        method = getattr(self._instance, "list_assets", None)
        if method is None:
            raise NotSupportedError()
        return method(refresh_reason, **kwargs)

    def add_asset(self, asset, **kwargs):
        method = getattr(self._instance, "add_asset", None)
        if method is None:
            raise NotSupportedError()
        return method(asset, **kwargs)

    def remove_asset(self, asset_identification, **kwargs):
        method = getattr(self._instance, "remove_asset", None)
        if method is None:
            raise NotSupportedError()
        return method(asset_identification, **kwargs)

    def update_asset(self, asset_update, **kwargs):
        method = getattr(self._instance, "update_asset", None)
        if method is None:
            raise NotSupportedError()
        return method(asset_update, **kwargs)

    def self_test(self, asset_identification, **kwargs):
        # kwargs for modes
        method = getattr(self._instance, "self_test", None)
        if method is None:
            raise NotSupportedError()
        return method(asset_identification, **kwargs)

    def self_calibrate(self, asset_identification, **kwargs):
        # kwargs for modes
        method = getattr(self._instance, "self_calibrate", None)
        if method is None:
            raise NotSupportedError()
        return method(asset_identification, **kwargs)

    def reset(self, asset_identification, **kwargs):
        # kwargs for modes
        method = getattr(self._instance, "reset", None)
        if method is None:
            raise NotSupportedError()
        return method(asset_identification, **kwargs)

    def refresh(self, **kwargs):
        # kwargs for sys api attributes
        method = getattr(self._instance, "refresh", None)
        if method is None:
            raise NotSupportedError()
        return method(**kwargs)

    def register_change_callback(self, callback_id, callback, *args):
        method = getattr(self._instance, "register_change_callback", None)
        if method is None:
            return
        method(callback_id, callback, *args)
